**CLASS**

# `HTTPNetworkTransport`

```swift
public class HTTPNetworkTransport
```

> A network transport that uses HTTP POST requests to send GraphQL operations to a server, and that uses `URLSession` as the networking implementation.

## Methods
### `init(url:session:sendOperationIdentifiers:useGETForQueries:enableAutoPersistedQueries:useGETForPersistedQueryRetry:delegate:requestCreator:)`

```swift
public init(url: URL,
            session: URLSession = .shared,
            sendOperationIdentifiers: Bool = false,
            useGETForQueries: Bool = false,
            enableAutoPersistedQueries: Bool = false,
            useGETForPersistedQueryRetry: Bool = false,
            delegate: HTTPNetworkTransportDelegate? = nil,
            requestCreator: RequestCreator = ApolloRequestCreator())
```

> Creates a network transport with the specified server URL and session configuration.
>
> - Parameters:
>   - url: The URL of a GraphQL server to connect to.
>   - session: The URLSession to use. Defaults to `URLSession.shared`,
>   - sendOperationIdentifiers: Whether to send operation identifiers rather than full operation text, for use with servers that support query persistence. Defaults to false.
>   - useGETForQueries: If query operation should be sent using GET instead of POST. Defaults to false.
>   - enableAutoPersistedQueries: Whether to send persistedQuery extension. QueryDocument will be absent at 1st request, retry with QueryDocument if server respond PersistedQueryNotFound or PersistedQueryNotSupport. Defaults to false.
>   - useGETForPersistedQueryRetry: Whether to retry persistedQuery request with HttpGetMethod. Defaults to false.
>   - delegate: [Optional] A delegate which can conform to any or all of `HTTPNetworkTransportPreflightDelegate`, `HTTPNetworkTransportTaskCompletedDelegate`, and `HTTPNetworkTransportRetryDelegate`. Defaults to nil.

#### Parameters

| Name | Description |
| ---- | ----------- |
| url | The URL of a GraphQL server to connect to. |
| session | The URLSession to use. Defaults to `URLSession.shared`, |
| sendOperationIdentifiers | Whether to send operation identifiers rather than full operation text, for use with servers that support query persistence. Defaults to false. |
| useGETForQueries | If query operation should be sent using GET instead of POST. Defaults to false. |
| enableAutoPersistedQueries | Whether to send persistedQuery extension. QueryDocument will be absent at 1st request, retry with QueryDocument if server respond PersistedQueryNotFound or PersistedQueryNotSupport. Defaults to false. |
| useGETForPersistedQueryRetry | Whether to retry persistedQuery request with HttpGetMethod. Defaults to false. |
| delegate | [Optional] A delegate which can conform to any or all of `HTTPNetworkTransportPreflightDelegate`, `HTTPNetworkTransportTaskCompletedDelegate`, and `HTTPNetworkTransportRetryDelegate`. Defaults to nil. |